#!/usr/bin/env python3
"""
Script to download NumPy wheels from the Anaconda staging area.

Usage::

    $ ./tools/download-wheels.py <version> -w <optional-wheelhouse>

The default wheelhouse is ``release/installers``.

Dependencies
------------

- beautifulsoup4
- urllib3

Examples
--------

While in the repository root::

    $ python tools/download-wheels.py 1.19.0
    $ python tools/download-wheels.py 1.19.0 -w ~/wheelhouse

"""
import argparse
import os
import re
import shutil

import urllib3
from bs4 import BeautifulSoup

__version__ = "0.2"

# Edit these for other projects.

# The first URL is used to get the file names as it avoids the need for paging
# when the number of files exceeds the page length. Note that files/page is not
# stable and can change when the page layout changes. The second URL is used to
# retrieve the files themselves. This workaround is copied from SciPy.
NAMES_URL = "https://pypi.anaconda.org/multibuild-wheels-staging/simple/numpy/"
FILES_URL = "https://anaconda.org/multibuild-wheels-staging/numpy"

# Name prefix of the files to download.
PREFIX = "numpy"

# Name endings of the files to download.
WHL = r"-.*\.whl$"
ZIP = r"\.zip$"
GZIP = r"\.tar\.gz$"
SUFFIX = rf"({WHL}|{GZIP}|{ZIP})"


def get_wheel_names(version):
    """ Get wheel names from Anaconda HTML directory.

    This looks in the Anaconda multibuild-wheels-staging page and
    parses the HTML to get all the wheel names for a release version.

    Parameters
    ----------
    version : str
        The release version. For instance, "1.18.3".

    """
    http = urllib3.PoolManager(cert_reqs="CERT_REQUIRED")
    tmpl = re.compile(rf"^.*{PREFIX}-{version}{SUFFIX}")
    index_url = f"{NAMES_URL}"
    index_html = http.request('GET', index_url)
    soup = BeautifulSoup(index_html.data, 'html.parser')
    return sorted(soup.find_all(string=tmpl))


def download_wheels(version, wheelhouse, test=False):
    """Download release wheels.

    The release wheels for the given NumPy version are downloaded
    into the given directory.

    Parameters
    ----------
    version : str
        The release version. For instance, "1.18.3".
    wheelhouse : str
        Directory in which to download the wheels.

    """
    http = urllib3.PoolManager(cert_reqs="CERT_REQUIRED")
    wheel_names = get_wheel_names(version)

    for i, wheel_name in enumerate(wheel_names):
        wheel_url = f"{FILES_URL}/{version}/download/{wheel_name}"
        wheel_path = os.path.join(wheelhouse, wheel_name)
        with open(wheel_path, "wb") as f:
            with http.request("GET", wheel_url, preload_content=False,) as r:
                info = r.info()
                length = int(info.get('Content-Length', '0'))
                if length == 0:
                    length = 'unknown size'
                else:
                    length = f"{(length / 1024 / 1024):.2f}MB"
                print(f"{i + 1:<4}{wheel_name} {length}")
                if not test:
                    shutil.copyfileobj(r, f)
    print(f"\nTotal files downloaded: {len(wheel_names)}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "version",
        help="NumPy version to download.")
    parser.add_argument(
        "-w", "--wheelhouse",
        default=os.path.join(os.getcwd(), "release", "installers"),
        help="Directory in which to store downloaded wheels\n"
             "[defaults to <cwd>/release/installers]")
    parser.add_argument(
        "-t", "--test",
        action='store_true',
        help="only list available wheels, do not download")

    args = parser.parse_args()

    wheelhouse = os.path.expanduser(args.wheelhouse)
    if not os.path.isdir(wheelhouse):
        raise RuntimeError(
            f"{wheelhouse} wheelhouse directory is not present."
            " Perhaps you need to use the '-w' flag to specify one.")

    download_wheels(args.version, wheelhouse, test=args.test)



import re
from collections.abc import Callable
from typing import (
    Any,
    ClassVar,
    Final,
    Generic,
    NamedTuple,
    TypeVar,
    final,
    type_check_only,
)
from typing import (
    Literal as L,
)

from typing_extensions import TypeIs

__all__ = ["VERSION_PATTERN", "InvalidVersion", "LegacyVersion", "Version", "parse"]

###

_CmpKeyT = TypeVar("_CmpKeyT", bound=tuple[object, ...])
_CmpKeyT_co = TypeVar("_CmpKeyT_co", bound=tuple[object, ...], default=tuple[Any, ...], covariant=True)

###

VERSION_PATTERN: Final[str] = ...

class InvalidVersion(ValueError): ...

@type_check_only
@final
class _InfinityType:
    def __hash__(self) -> int: ...
    def __eq__(self, other: object, /) -> TypeIs[_InfinityType]: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __lt__(self, other: object, /) -> L[False]: ...
    def __le__(self, other: object, /) -> L[False]: ...
    def __gt__(self, other: object, /) -> L[True]: ...
    def __ge__(self, other: object, /) -> L[True]: ...
    def __neg__(self) -> _NegativeInfinityType: ...

Infinity: Final[_InfinityType] = ...

@type_check_only
@final
class _NegativeInfinityType:
    def __hash__(self) -> int: ...
    def __eq__(self, other: object, /) -> TypeIs[_NegativeInfinityType]: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __lt__(self, other: object, /) -> L[True]: ...
    def __le__(self, other: object, /) -> L[True]: ...
    def __gt__(self, other: object, /) -> L[False]: ...
    def __ge__(self, other: object, /) -> L[False]: ...
    def __neg__(self) -> _InfinityType: ...

NegativeInfinity: Final[_NegativeInfinityType] = ...

class _Version(NamedTuple):
    epoch: int
    release: tuple[int, ...]
    dev: tuple[str, int] | None
    pre: tuple[str, int] | None
    post: tuple[str, int] | None
    local: tuple[str | int, ...] | None

class _BaseVersion(Generic[_CmpKeyT_co]):
    _key: _CmpKeyT_co
    def __hash__(self) -> int: ...
    def __eq__(self, other: _BaseVersion, /) -> bool: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def __ne__(self, other: _BaseVersion, /) -> bool: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def __lt__(self, other: _BaseVersion, /) -> bool: ...
    def __le__(self, other: _BaseVersion, /) -> bool: ...
    def __ge__(self, other: _BaseVersion, /) -> bool: ...
    def __gt__(self, other: _BaseVersion, /) -> bool: ...
    def _compare(self, /, other: _BaseVersion[_CmpKeyT], method: Callable[[_CmpKeyT_co, _CmpKeyT], bool]) -> bool: ...

class LegacyVersion(_BaseVersion[tuple[L[-1], tuple[str, ...]]]):
    _version: Final[str]
    def __init__(self, /, version: str) -> None: ...
    @property
    def public(self) -> str: ...
    @property
    def base_version(self) -> str: ...
    @property
    def local(self) -> None: ...
    @property
    def is_prerelease(self) -> L[False]: ...
    @property
    def is_postrelease(self) -> L[False]: ...

class Version(
    _BaseVersion[
        tuple[
            int,  # epoch
            tuple[int, ...],  # release
            tuple[str, int] | _InfinityType | _NegativeInfinityType,  # pre
            tuple[str, int] | _NegativeInfinityType,  # post
            tuple[str, int] | _InfinityType,  # dev
            tuple[tuple[int, L[""]] | tuple[_NegativeInfinityType, str], ...] | _NegativeInfinityType,  # local
        ],
    ],
):
    _regex: ClassVar[re.Pattern[str]] = ...
    _version: Final[str]

    def __init__(self, /, version: str) -> None: ...
    @property
    def public(self) -> str: ...
    @property
    def base_version(self) -> str: ...
    @property
    def local(self) -> str | None: ...
    @property
    def is_prerelease(self) -> bool: ...
    @property
    def is_postrelease(self) -> bool: ...

#
def parse(version: str) -> Version | LegacyVersion: ...
